//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useEffect, useState } from "react";
import Button from "../../controls/action-button";
import { COLORS, doNothing, isNonEmptyString } from "primeobjects-helper-util";
import { GradientDiv, SVG } from "primeobjects-ui-web-basic";
import Table from "../../controls/antd/table";
import Tooltip from "../../controls/antd/tooltip";
import { map, maxBy, isNil } from "lodash";
import { getAIExtractorFileName, startAIExtractorJob } from "./ai-extractor-util";
import Switch from "../../controls/antd/switch";
import RadioGroup from "../../controls/antd/radio-group";
import RadioButton from "../../controls/antd/radio-button";
var IDENTIFIER_INTRO = "Identifier will be submitted in the extracted data. \nWebhook can identify the type/source of the data by using the value in the identifier. \nFor example, entity name such as 'User' can be used as indentifier so the webhook will handle the data as a user profile.";
var MappingInputInternal = function (props) {
    var onChange = props.onChange, color = props.color;
    var _a = useState(undefined), value = _a[0], setValue = _a[1];
    useEffect(function () {
        if (props.value != value)
            setValue(props.value);
    }, [props.value]);
    return (React.createElement("div", { className: "w-full flex flex-row" },
        React.createElement("div", { className: "flex items-center" },
            React.createElement(SVG, { src: "/icons/platform/right.svg", size: 16 })),
        React.createElement("input", { placeholder: "Map to the field name ...", className: "flex-1 border px-1 ml-1 text-".concat(color.name, "-500"), style: { width: 140 }, value: value, onChange: function (e) {
                setValue(e.target.value);
                onChange(e.target.value);
            } })));
};
MappingInputInternal.displayName = "MappingInput";
var MappingInput = React.memo(MappingInputInternal);
export var AIExtractorFormForCustomization = function (props) {
    var entity = props.entity, hideTabs = props.hideTabs, onGoBack = props.onGoBack, record = props.record;
    var FormWrapper = props.FormWrapper;
    var getGeneralForm = function (customProps) {
        var solution = customProps.solution, entity = customProps.entity, isError = customProps.isError, isDoing = customProps.isDoing, version = customProps.version, disabled = customProps.disabled, onStartAPICall = customProps.onStartAPICall, onSuccessAPICall = customProps.onSuccessAPICall, toChangeRecord = customProps.toChangeRecord, onErrorAPICall = customProps.onErrorAPICall, onError = customProps.onError, primaryColor = customProps.primaryColor;
        var getRows = function (record) {
            var currentResultView = record.currentResultView
                ? record.currentResultView
                : "fields";
            var onChangeMapping = function (key, fieldName) {
                if (!record.mapping)
                    record.mapping = {};
                if (isNonEmptyString(fieldName)) {
                    record.mapping[key] = fieldName;
                }
                else {
                    delete record.mapping[key];
                }
                updateRecord(record);
            };
            var onChangeTableKey = function (row, rowKey, cIndex) {
                if (!record.tableKeys)
                    record.tableKeys = {};
                //each row can only have one key field
                //remove all columns of the row first
                for (var i = 0; i < row.cellsCount; i++) {
                    delete record.tableKeys[row["cell".concat(i + 1, "Key")]];
                }
                if (isNonEmptyString(cIndex)) {
                    record.tableKeys[rowKey] = cIndex;
                }
                updateRecord(record);
            };
            var getRowKey = function (row) {
                if (!record.tableKeys)
                    record.tableKeys = {};
                //each row can only have one key field
                //remove all columns of the row first
                for (var i = 0; i < row.cellsCount; i++) {
                    var rowKey = row["cell".concat(i + 1, "Key")];
                    if (record.tableKeys[rowKey])
                        return rowKey;
                }
                return row.key;
            };
            var getColumnsForFields = function () {
                return [
                    {
                        dataIndex: "key",
                        width: 120,
                        title: "Key",
                        fixed: "left",
                        render: function (v) {
                            return (React.createElement(Tooltip, { placement: "bottom", title: v },
                                React.createElement("div", { className: "line-clamp-2" }, v)));
                        }
                    },
                    {
                        dataIndex: "label",
                        width: 185,
                        title: "Label",
                        render: function (label, row) {
                            var key = "".concat(row.key, "_LABEL");
                            return (React.createElement(React.Fragment, null,
                                React.createElement(Tooltip, { placement: "bottom", title: label },
                                    React.createElement(GradientDiv, { className: "w-full h-5 mb-2 pl-5" }, label)),
                                React.createElement(MappingInput, { color: primaryColor, value: (record === null || record === void 0 ? void 0 : record.mapping) &&
                                        (record === null || record === void 0 ? void 0 : record.mapping[key])
                                        ? record === null || record === void 0 ? void 0 : record.mapping[key]
                                        : "", onChange: function (v) {
                                        return onChangeMapping(key, v);
                                    } })));
                        }
                    },
                    {
                        dataIndex: "value",
                        width: 185,
                        title: "Value",
                        render: function (value, row) {
                            var key = "".concat(row.key, "_VALUE");
                            return (React.createElement(React.Fragment, null,
                                React.createElement(Tooltip, { placement: "bottom", title: value },
                                    React.createElement(GradientDiv, { className: "w-full h-5 mb-2 pl-5" }, value)),
                                React.createElement(MappingInput, { color: primaryColor, value: (record === null || record === void 0 ? void 0 : record.mapping) &&
                                        (record === null || record === void 0 ? void 0 : record.mapping[key])
                                        ? record === null || record === void 0 ? void 0 : record.mapping[key]
                                        : "", onChange: function (v) {
                                        return onChangeMapping(key, v);
                                    } })));
                        }
                    }
                ];
            };
            var getColumnsForRows = function (rows) {
                var maxByRow = maxBy(rows, "cellsCount");
                var count = maxByRow ? maxByRow.cellsCount : 1;
                return __spreadArray([
                    {
                        dataIndex: "key",
                        width: 120,
                        title: "Key",
                        render: function (v, row) {
                            doNothing(v);
                            var key = getRowKey(row);
                            return (React.createElement(Tooltip, { placement: "bottom", title: key },
                                React.createElement("div", { className: "line-clamp-2" }, key)));
                        },
                        fixed: "left"
                    }
                ], map(new Array(count), function (c, index) {
                    doNothing(c);
                    return {
                        dataIndex: "cell".concat(index + 1),
                        width: 185,
                        title: "C".concat(index + 1),
                        render: function (v, row) {
                            var cellRowKey = row["cell".concat(index + 1, "Key")];
                            var curRowKey = getRowKey(row);
                            var mappingKey = "".concat(curRowKey, "_C").concat(index + 1);
                            return (React.createElement(React.Fragment, null,
                                React.createElement("div", { className: "w-full flex flex-row" },
                                    React.createElement(Tooltip, { placement: "bottom", title: "Is key value" },
                                        React.createElement(Switch, { style: { marginTop: 3 }, size: "small", checked: !isNil((record === null || record === void 0 ? void 0 : record.tableKeys) &&
                                                (record === null || record === void 0 ? void 0 : record.tableKeys[cellRowKey])), onChange: function (switchValue) {
                                                onChangeTableKey(row, cellRowKey, switchValue
                                                    ? "C".concat(index + 1)
                                                    : "");
                                            } })),
                                    React.createElement(Tooltip, { placement: "bottom", title: v },
                                        React.createElement(GradientDiv, { className: "h-5 mb-2 pl-1" }, v))),
                                React.createElement(MappingInput, { color: primaryColor, value: (record === null || record === void 0 ? void 0 : record.mapping) &&
                                        (record === null || record === void 0 ? void 0 : record.mapping[mappingKey])
                                        ? record === null || record === void 0 ? void 0 : record.mapping[mappingKey]
                                        : "", onChange: function (v) {
                                        return onChangeMapping(mappingKey, v);
                                    } })));
                        }
                    };
                }), true);
            };
            var updateRecord = function (newRecord) {
                record = newRecord;
                toChangeRecord(record);
            };
            var clearAllError = function () {
                onError("ai", false);
            };
            var renderMappings = function () {
                return record.schemaResultForFields ? (React.createElement("div", { className: "w-full border" },
                    React.createElement(Table, { pagination: false, rowKey: "index", columns: [
                            {
                                dataIndex: "key",
                                width: 300,
                                title: "Key",
                                fixed: "left",
                                render: function (key) {
                                    return (React.createElement(Tooltip, { placement: "bottom", title: key },
                                        React.createElement("div", { className: "line-clamp-2" }, key)));
                                }
                            },
                            {
                                dataIndex: "value",
                                title: "Field Name",
                                fixed: "left",
                                render: function (v) {
                                    return (React.createElement(Tooltip, { placement: "bottom", title: v },
                                        React.createElement(GradientDiv, { className: "w-full" }, v)));
                                }
                            }
                        ], scroll: { x: "100%" }, dataSource: map(map(record.mapping, function (value, key) {
                            return { key: key, value: value };
                        }), function (r, index) {
                            return __assign(__assign({}, r), { index: index });
                        }) }))) : (React.createElement(React.Fragment, null));
            };
            var renderResult = function (type) {
                if (!record.schemaResultForFields)
                    return React.createElement(React.Fragment, null);
                switch (type) {
                    case "rows":
                        return renderResultForRows();
                    case "fields":
                        return renderResultForFields();
                    default:
                        return React.createElement(React.Fragment, null);
                }
            };
            var renderResultForRows = function () {
                return (React.createElement("div", { className: "w-full border ".concat(currentResultView == "rows" ? "" : "hidden") },
                    React.createElement(Table, { rowKey: "index", pagination: false, columns: getColumnsForRows(record.schemaResultForRows), scroll: { x: "100%", y: 500 }, dataSource: record.schemaResultForRows })));
            };
            var renderResultForFields = function () {
                return (React.createElement("div", { className: "w-full border ".concat(currentResultView == "fields" ? "" : "hidden") },
                    React.createElement(Table, { rowKey: "index", pagination: false, columns: getColumnsForFields(), scroll: { x: "100%", y: 500 }, dataSource: record.schemaResultForFields })));
            };
            var onClickSubmitButton = function () {
                startAIExtractorJob(solution, record, "schema", updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall);
            };
            var onClickClearData = function () {
                delete record.resultForFields;
                delete record.resultForRows;
                delete record.schemaResultForFields;
                delete record.schemaResultForRows;
                delete record.taskResultForFields;
                delete record.taskResultForRows;
                delete record.mapping;
                delete record.tableKeys;
                updateRecord(record);
            };
            var onChangeResultView = function (viewType) {
                record.currentResultView = viewType;
                updateRecord(record);
            };
            return [
                !isDoing("ai") && {
                    fields: [
                        {
                            name: "isPublished",
                            type: "checkbox",
                            label: "The template is ready to use",
                            defaultValue: false,
                            size: "small",
                            disabled: isDoing("ai"),
                            colStyle: { width: 300, flex: "none" }
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "name",
                            label: "Name",
                            alwaysShowLabel: true,
                            type: "single-line-input",
                            disabled: isDoing("ai"),
                            placeholder: "Type the name of the ".concat(entity.uiName, " here")
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "description",
                            label: "Description",
                            alwaysShowLabel: true,
                            type: "multi-line-input",
                            disabled: isDoing("ai"),
                            placeholder: "Type the description of the ".concat(entity.uiName, " here")
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "ai",
                            line: "Upload file & define the data mapping",
                            type: "section"
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "schemaFileUrl",
                            label: "Upload a file to define schema",
                            alwaysShowLabel: true,
                            fileType: "Extract",
                            subNote: "Supported format: pdf, jpeg and jpg",
                            type: "upload",
                            disabled: isDoing("ai"),
                            onChange: function (newRecord) {
                                clearAllError();
                                var fileName = getAIExtractorFileName("schema", newRecord);
                                delete newRecord.schemaJobId;
                                delete newRecord.schemaResultForFields;
                                delete newRecord.schemaResultForRows;
                                delete newRecord.schemaResult;
                                if (fileName) {
                                    return __assign(__assign({}, newRecord), { fileName: fileName });
                                }
                                return newRecord;
                            }
                        }
                    ]
                },
                !isDoing("ai") &&
                    isNonEmptyString(record.schemaFileUrl) && {
                    fields: [
                        {
                            name: "ai",
                            content: (React.createElement("div", { className: "flex flex-row" },
                                React.createElement(Button, { id: "ai-submit-button", wrapperStyle: { marginLeft: 0 }, onClick: onClickSubmitButton, wrapperClassName: "bg-sky-400 hover:bg-sky-500 justify-center px-2", text: "Submit to AI", textColor: COLORS.WHITE }),
                                React.createElement(Button, { id: "ai-clear-data-button", onClick: onClickClearData, wrapperClassName: "bg-red-400 hover:bg-red-500 justify-center px-2", text: "Clear data & mappings", textColor: COLORS.WHITE }))),
                            type: "container"
                        }
                    ]
                },
                isError("ai") && {
                    fields: [
                        {
                            name: "aiFailed",
                            type: "alert-error",
                            line: "Sorry, we failed to process the uploaded file. Please try again later."
                        }
                    ]
                },
                isDoing("ai") && {
                    fields: [
                        {
                            name: "aiDoing",
                            content: (React.createElement("div", { className: "w-full flex flex-row" },
                                React.createElement(SVG, { src: "/icons/platform/loading.svg", className: "spinner", style: { width: 22, height: 22 }, color: COLORS.SKY["600"] }),
                                React.createElement("span", { className: "pl-2 text-sky-600" }, "Processing the file ..."))),
                            type: "container"
                        }
                    ]
                },
                !isDoing("ai") && {
                    fields: [
                        {
                            label: "Mappings",
                            name: "mappings",
                            type: "container",
                            content: renderMappings()
                        }
                    ]
                },
                !isDoing("ai") &&
                    isNonEmptyString(record.schemaFileUrl) && {
                    fields: [
                        {
                            name: "selectToViewResult",
                            content: (React.createElement("div", { className: "flex flex-col w-full" },
                                React.createElement("div", { className: "flex flex-row mb-2" },
                                    React.createElement(RadioGroup, { value: currentResultView, color: primaryColor["600"], onChange: function (e) {
                                            return onChangeResultView(e.target.value);
                                        } },
                                        React.createElement(RadioButton, { value: "fields" }, "View fields"),
                                        React.createElement(RadioButton, { value: "rows" }, "View rows"))),
                                renderResult("fields"),
                                renderResult("rows"))),
                            type: "container"
                        }
                    ]
                }
            ];
        };
        return {
            id: "general",
            tooltip: "General",
            iconSrc: "/icons/platform/form.svg",
            getRows: getRows,
            version: version,
            disabled: disabled == true
        };
    };
    var getWebHookForm = function (customProps) {
        var entity = customProps.entity, isDoing = customProps.isDoing, version = customProps.version, disabled = customProps.disabled, toShow = customProps.toShow, onShow = customProps.onShow;
        var getRows = function (record) {
            doNothing(record);
            return [
                toShow("identifier") && {
                    fields: [
                        {
                            name: "identifierNote",
                            type: "alert-info",
                            line: IDENTIFIER_INTRO
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "identifier",
                            label: "Identifier",
                            alwaysShowLabel: true,
                            type: "single-line-input",
                            placeholder: "Type the identifier of the ".concat(entity.uiName, " here"),
                            disabled: isDoing("ai"),
                            onClickInfo: function () {
                                onShow("identifier", !toShow("identifier"));
                            }
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "devWebhook",
                            label: "Webhook endpoint - DEV Enviroment (POST)",
                            alwaysShowLabel: true,
                            type: "single-line-input",
                            placeholder: "Type the url here",
                            format: "url",
                            disabled: isDoing("ai"),
                            onClickInfo: function () {
                                onShow("identifier", !toShow("identifier"));
                            }
                        }
                    ]
                },
                {
                    fields: [
                        {
                            name: "prodWebhook",
                            label: "Webhook endpoint - PROD Enviroment (POST)",
                            alwaysShowLabel: true,
                            type: "single-line-input",
                            placeholder: "Type the url here",
                            format: "url",
                            disabled: isDoing("ai"),
                            onClickInfo: function () {
                                onShow("identifier", !toShow("identifier"));
                            }
                        }
                    ]
                }
            ];
        };
        return {
            id: "webhook",
            tooltip: "Webhook",
            iconSrc: "/icons/platform/webhook.svg",
            getRows: getRows,
            version: version,
            disabled: disabled == true
        };
    };
    return (React.createElement(FormWrapper, __assign({}, props, { tabs: [getGeneralForm, getWebHookForm], entity: entity, record: record, hideActionButton: true, hideTabs: hideTabs, onGoBack: onGoBack })));
};
//# sourceMappingURL=ai-extractor-for-customization.js.map