//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect, useState } from "react";
import ListSectionInternal from "../sections/list/index";
import ListTreeSectionInternal from "../sections/list/tree";
import { observer } from "mobx-react-lite/dist/index";
import { useEnvStore } from "primeobjects-ui-web-store/build/cjs/stores/env";
import { isNonEmptyString } from "primeobjects-helper-util/build/cjs/core";
import ListNavSectionInternal, { CATEGORIES_OPTION, TAGS_OPTION, LISTS_OPTION } from "../sections/list/nav";
import { cloneDeep, each, filter, isEmpty, isNumber } from "lodash";
import { ResizeDetectorDiv } from "primeobjects-ui-web-basic/build/cjs/controls/resize-detector-div";
import { doNothing } from "primeobjects-helper-util";
import { getGlobalComponent } from "../util";
import { getPrimaryColor, useSolution } from "primeobjects-helper-ui";
import { COLORS, hexToRgbaString } from "primeobjects-helper-util/build/cjs/colors";
import { useCurrentContext } from "../context/use-current-context";
var ListSection = undefined;
var ListTreeSection = undefined;
var ListNavSection = undefined;
var FormWrapperFromGlobal = undefined;
var SingleLineInputField = undefined;
var TreeField = undefined;
export var ListPage = observer(function (props) {
    if (!ListSection) {
        console.log("Get global components in /pages/list.tsx");
        ListSection = getGlobalComponent("ListSection", ListSectionInternal);
        ListTreeSection = getGlobalComponent("ListTreeSection", ListTreeSectionInternal);
        ListNavSection = getGlobalComponent("ListNavSection", ListNavSectionInternal);
        FormWrapperFromGlobal = getGlobalComponent("FormWrapper");
        SingleLineInputField = getGlobalComponent("SingleLineInputField");
        TreeField = getGlobalComponent("TreeField");
    }
    var apis = props.apis, defaultRecordId = props.defaultRecordId;
    var solution = useSolution();
    var currentContext = useCurrentContext(solution);
    var organizationId = currentContext.organizationId;
    var primaryColor = getPrimaryColor();
    var FormWrapper = props.Form;
    var listViewMaxWidth = isNumber(props.listViewMaxWidth)
        ? props.listViewMaxWidth
        : 950;
    var formViewMaxWidth = isNumber(props.formViewMaxWidth)
        ? props.formViewMaxWidth
        : 800;
    var _a = useState(false), closeDrawer = _a[0], setCloseDrawer = _a[1];
    var _b = useState(false), closeListArea = _b[0], setCloseListArea = _b[1];
    var _c = useState(false), openInSmall = _c[0], setOpenInSmall = _c[1];
    var _d = useState(""), error = _d[0], setError = _d[1];
    var _e = useState(0), wrapperWidth = _e[0], setWrapperWidth = _e[1];
    var _f = useState([]), navOptions = _f[0], setNavOptions = _f[1];
    var _g = useState(undefined), curFormTab = _g[0], setCurFormTab = _g[1];
    var _h = useState(undefined), navOption = _h[0], setNavOption = _h[1];
    var _j = useState({}), curRecord = _j[0], setCurRecord = _j[1];
    var _k = useState([]), nodes = _k[0], setNodes = _k[1];
    var _l = useState([]), nodesSelected = _l[0], setNodesSelected = _l[1];
    var _m = useState(undefined), resetUI = _m[0], setResetUI = _m[1];
    var _o = useState(undefined), curDeletedRecord = _o[0], setCurDeletedRecord = _o[1];
    var _p = useState(undefined), curSavedRecord = _p[0], setSavedRecord = _p[1];
    var _q = useState(undefined), removeFilteredNode = _q[0], setRemoveFilteredNode = _q[1];
    var env = useEnvStore();
    // const [width, setWidth] = useState(0);
    // const [dataAreaWidth, setDataAreaWidth] = useState(0);
    var smallScreen = env.size != "xl" && env.size != "2xl";
    var showDrawer = smallScreen && openInSmall && !closeDrawer;
    var showListArea = (!smallScreen && !closeListArea) || showDrawer;
    var listSectionProps = props.listSectionProps;
    var formWrapperProps = props.formWrapperProps;
    var view = props.view ? props.view : "list-form";
    var height = env.height - (props.adjHeight ? props.adjHeight : 0);
    useEffect(function () {
        if (props.entity.entityName != (navOption === null || navOption === void 0 ? void 0 : navOption.regardingEntityName)) {
            setCurRecord({});
            setSavedRecord(undefined);
            setError("");
            setResetUI(Date.now());
            var newNavOptions = props.navOptions && props.navOptions.length > 0
                ? props.navOptions
                : [CATEGORIES_OPTION, TAGS_OPTION, LISTS_OPTION];
            setNavOptions(newNavOptions);
            var newNavOption = newNavOptions[0];
            setNavOption(__assign(__assign({}, newNavOption), { regardingEntityName: props.entity.entityName, regardingEntityType: props.entity.entityType }));
        }
        else {
            setError("The entity was not defined.");
        }
    }, [props.entity]);
    useEffect(function () {
        if (props.currentRecord && !isEmpty(props.currentRecord)) {
            setCurRecord(props.currentRecord);
        }
    }, [props.currentRecord]);
    // useEffect(() => {
    //     setDataAreaWidth();
    // }, [showListArea, showDrawer, width]);
    var onSelectOption = function (newNavOption) {
        setOpenInSmall(smallScreen);
        setNavOption(__assign(__assign({}, newNavOption), { regardingEntityName: props.entity.entityName, regardingEntityType: props.entity.entityType }));
        setCloseDrawer(false);
    };
    var onCloseDrawer = function () {
        setCloseDrawer(true);
    };
    var onToggleClose = function (closed) {
        setCloseListArea(closed);
    };
    var onEditRecord = function (record, action, entity, rowIndex) {
        doNothing({ action: action, entity: entity, rowIndex: rowIndex });
        var newRecord = (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onEditRecord) &&
            (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onEditRecord(record));
        setCurRecord(cloneDeep(newRecord !== null && newRecord !== void 0 ? newRecord : record));
    };
    var onClickRecord = function (record, action, entity, rowIndex) {
        var result = props.onClickRecord
            ? props.onClickRecord(record, action, entity, rowIndex)
            : (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onClickRecord)
                ? listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onClickRecord(record, action, entity, rowIndex)
                : undefined;
        if (result) {
            each(result, function (r) {
                onActionRecord(r.actionName, r.actionData);
            });
        }
    };
    var onCreateRecord = function (record) {
        var newRecord = (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onCreateRecord) &&
            (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onCreateRecord(record));
        setCurRecord(newRecord !== null && newRecord !== void 0 ? newRecord : record);
        setCurFormTab("");
    };
    var onSaveRecord = function (record) {
        setCurRecord(record);
        setSavedRecord(record);
    };
    var renderForm = function (onGoBack) {
        return (React.createElement(FormWrapper, __assign({}, formWrapperProps, { FormWrapper: FormWrapperFromGlobal, wrapperClassName: "w-full", currentTab: curFormTab, entity: props.entity, record: curRecord, onGoBack: onGoBack, onSave: onSaveRecord, onDelete: onDeleteRecord, onAction: onActionRecord, onClone: onCloneRecord })));
    };
    var onCloneRecord = function (record) {
        onSaveRecord(record);
    };
    var onActionRecord = function (name, data) {
        switch (name) {
            case "change-tab": {
                setCurFormTab("".concat(data, "|").concat(Date.now()));
                break;
            }
            case "update-current-record": {
                setCurRecord(data);
                setCurFormTab("|".concat(Date.now()));
                break;
            }
        }
    };
    var onDeleteRecord = function (record) {
        var newRecord = (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onDeleteRecord) &&
            (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.onDeleteRecord(record));
        setCurRecord({});
        setCurDeletedRecord(newRecord !== null && newRecord !== void 0 ? newRecord : record);
    };
    var onResizeWrapper = function (width) {
        setWrapperWidth(width);
        props.onResizeWrapper && props.onResizeWrapper(width, props);
    };
    var renderContent = function () {
        var _a;
        if (!props.entity || isNonEmptyString(error)) {
            return (React.createElement("div", { className: "w-full text-red-600 p-6 border-l border-r", style: { height: height } }, error));
        }
        switch ((_a = props.entity) === null || _a === void 0 ? void 0 : _a.entityName) {
            case "Dashboard": {
                return (React.createElement("div", { className: "w-full flex flex-row", style: { height: height } }));
            }
            default: {
                return (React.createElement(React.Fragment, null,
                    renderListOnlyView(),
                    renderListFormView(),
                    renderTreeListView()));
            }
        }
    };
    var getQuery = function () {
        var query = (listSectionProps === null || listSectionProps === void 0 ? void 0 : listSectionProps.query)
            ? listSectionProps.query
            : {};
        if (!query.entityName && props.entity) {
            query.entityName = props.entity.entityName;
            query.entityType = props.entity.entityType;
        }
        return query;
    };
    var renderListSection = function () {
        return (React.createElement(ListSection, __assign({}, listSectionProps, { defaultRecordId: defaultRecordId, onCloneRecord: onCloneRecord, recordToUpdate: curSavedRecord, recordDeleted: curDeletedRecord, onEditRecord: onEditRecord, onClickRecord: onClickRecord, onCreateRecord: onCreateRecord, renderCategories: view != "tree-list" && renderTree, nodes: nodes, currentRecord: curRecord, onRemoveFilter: onRemoveFilter, apis: apis, query: getQuery(), resetUI: resetUI, hideInfoHeader: props.hideListInfoHeader, infoHeaderContent: props.listInfoHeaderContent, hideInfoFooter: props.hideListInfoFooter, infoFooterContent: props.listInfoFooterContent }), props.children));
    };
    var renderList = function () {
        if (!props.entity)
            return React.createElement(React.Fragment, null);
        return (React.createElement(ResizeDetectorDiv, { onDetect: onResizeWrapper, className: "flex-1 flex flex-col bg-white overflow-hidden h-full max-h-full shadow rounded-lg", style: { minWidth: 400 } },
            React.createElement("div", { className: "list-wrapper h-full ".concat(!curRecord.id ? "" : "hidden") }, renderListSection()),
            React.createElement("div", { className: "form-wrapper h-full flex ".concat(curRecord.id ? "" : "hidden"), style: { maxWidth: formViewMaxWidth } }, renderForm(function () { return setCurRecord({}); }))));
    };
    var renderListFormView = function () {
        if (!props.entity || view != "list-form")
            return React.createElement(React.Fragment, null);
        var hideForm = !curRecord.id;
        return (React.createElement(ResizeDetectorDiv, { className: "list-form-view w-full flex flex-row list-view p-6", onDetect: onResizeWrapper, style: { height: height } },
            React.createElement("div", { className: "list-wrapper ".concat(hideForm ? "" : "mr-6", " h-full w-full bg-white rounded-lg shadow overflow-hidden \n                    ").concat(wrapperWidth < listViewMaxWidth ? "" : "max-w-md", " \n                    ").concat(wrapperWidth < listViewMaxWidth && curRecord.id
                    ? "hidden"
                    : "") }, renderListSection()),
            React.createElement("div", { className: "w-full flex flex-row overflow-hidden h-full max-h-full shadow rounded-lg ".concat(hideForm ? "hidden" : "") },
                React.createElement("div", { className: "form-wrapper h-full w-full bg-white" }, renderForm(wrapperWidth < listViewMaxWidth && curRecord.id
                    ? function () { return setCurRecord({}); }
                    : undefined)))));
    };
    var onClickTreeFilter = function (newNodes) {
        if (view != "tree-list") {
            setNodesSelected(filter(newNodes, function (n) { return !n.remove; }));
        }
        else {
            setNodes(filter(newNodes, function (n) { return !n.remove; }));
        }
    };
    var onClickApplyTreeFilter = function () {
        setNodes(nodesSelected);
    };
    var onRemoveFilter = function (filter, newData) {
        switch (filter.type) {
            case "node": {
                setNodes(newData ? newData : []);
                setRemoveFilteredNode(filter);
                break;
            }
        }
    };
    var renderTree = function (onClose) {
        var _a;
        if ((navOption === null || navOption === void 0 ? void 0 : navOption.type) != "tree" ||
            ((navOption === null || navOption === void 0 ? void 0 : navOption.type) == "tree" &&
                !(navOption.attributeName && navOption.entityName))) {
            return (React.createElement("div", { className: "w-full text-red-600 p-4" }, "Please define entityName and attributeName."));
        }
        return (React.createElement("div", { className: "flex flex-col w-full", style: {
                height: view != "tree-list" ? height - 180 : "100%"
            } },
            React.createElement("div", { className: "w-full", style: {
                    height: view != "tree-list" ? height - 220 : "100%"
                } },
                React.createElement(ListTreeSection, { wrapperClassName: navOption.type == "tree" ? "" : "hidden", TreeField: TreeField, SingleLineInputField: SingleLineInputField, organizationId: (_a = navOption.organizationId) !== null && _a !== void 0 ? _a : organizationId, attributeName: navOption.attributeName, entityName: navOption.entityName, readonly: navOption.readonly, showFilterButton: navOption.showFilterButton, regardingEntityName: navOption.regardingEntityName, regardingEntityType: navOption.regardingEntityType, onClickFilter: onClickTreeFilter, removeFilteredNode: removeFilteredNode })),
            view != "tree-list" && (React.createElement("div", { className: "flex flex-row w-full p-2 " },
                React.createElement("div", { className: "flex-1 py-4" }),
                React.createElement("div", { onClick: function () {
                        onClose && onClose();
                    }, className: "flex sticky border rounded-md cursor-pointer py-2 px-3 shadow hover:bg-gray-50 hover:shadow-lg justify-center items-center leading-none mr-2" }, "Close"),
                React.createElement("div", { onClick: function () {
                        onClickApplyTreeFilter();
                        onClose && onClose();
                    }, className: "flex sticky rounded-md cursor-pointer py-2 px-3 shadow bg-".concat(primaryColor.name, "-400 hover:bg-").concat(primaryColor.name, "-500 text-white hover:shadow-lg justify-center items-center leading-none") }, "Apply Filter")))));
    };
    var renderTreeListView = function () {
        if (!props.entity || view != "tree-list" || !navOption)
            return React.createElement(React.Fragment, null);
        return (React.createElement("div", { className: "tree-list-view w-full flex flex-row tree-view p-6", style: { height: height } },
            React.createElement("div", { className: "flex bg-white h-full mr-6 overflow-hidden shadow rounded-lg" },
                React.createElement(ListNavSection, { currentId: showListArea ? navOption.id : undefined, onSelectOption: onSelectOption, showDrawer: showDrawer, options: navOptions, smallScreen: smallScreen, onCloseDrawer: smallScreen ? onCloseDrawer : function () { }, onToggleClose: onToggleClose, wrapperStyle: showDrawer
                        ? {
                            position: "fixed",
                            top: 0,
                            left: 0,
                            zIndex: "50"
                        }
                        : {
                            position: "relative",
                            display: (!curRecord.id && smallScreen) ||
                                !smallScreen
                                ? "flex"
                                : "none"
                        } }),
                React.createElement("div", { className: "list-tree-wrapper h-full bg-white", style: __assign({ width: 358, minWidth: 358, display: showListArea ? "flex" : "none" }, (showDrawer
                        ? {
                            position: "fixed",
                            top: 0,
                            left: 60,
                            zIndex: "50"
                        }
                        : { position: "relative" })) }, renderTree())),
            renderList(),
            showDrawer && (React.createElement("div", { className: "h-full max-h-full w-full z-10", style: {
                    position: "fixed",
                    left: 0,
                    right: 0,
                    background: hexToRgbaString(COLORS.BLACK, 0.1)
                } }))));
    };
    var renderListOnlyView = function () {
        if (!props.entity || view != "full-list" || !navOption)
            return React.createElement(React.Fragment, null);
        return (React.createElement("div", { className: "full-list-view w-full flex flex-row list-view p-4", style: { height: height } }, renderList()));
    };
    return renderContent();
});
export default ListPage;
//# sourceMappingURL=list.js.map